﻿// Kerbal Engineer Redux
// Author:  CYBUTEK
// License: Attribution-NonCommercial-ShareAlike 3.0 Unported

using System;
using System.Collections.Generic;
using System.Text;
using UnityEngine;
using KSP.IO;

namespace Engineer
{
    class Settings
    {
        Dictionary<string, string> settings = new Dictionary<string, string>();
        bool changed = false;

        public bool Changed
        {
            get
            {
                if (changed)
                {
                    changed = false;
                    return true;
                }
                else
                {
                    return false;
                }
            }
        }

        public void Load(string filename)
        {
            if (File.Exists<Settings>(filename))
            {
                settings.Clear();

                string[] lines = File.ReadAllLines<Settings>(filename);

                for (int i = 0; i < lines.Length; i++)
                {
                    string[] line = lines[i].Split('=');
                    settings.Add(line[0].Trim(), line[1].Trim());
                }
            }
        }

        public void Save(string filename)
        {
            if (settings.Count > 0)
            {
                TextWriter file = File.CreateText<Settings>(filename);

                foreach (KeyValuePair<string, string> setting in settings)
                {
                    file.WriteLine(setting.Key + " = " + setting.Value);
                }

                file.Close();
            }
        }

        public string Get(string key, string value = null)
        {
            if (settings.ContainsKey(key))
            {
                return settings[key];
            }
            else
            {
                Set(key, value);
                return value;
            }
        }

        public T Get<T>(string key)
        {
            if (settings.ContainsKey(key))
            {
                return (T)Convert.ChangeType(settings[key], typeof(T));
            }
            else
            {
                return (T)Convert.ChangeType(null, typeof(T));
            }
        }

        public T Get<T>(string key, T value)
        {
            if (settings.ContainsKey(key))
            {
                return (T)Convert.ChangeType(settings[key], typeof(T));
            }
            else
            {
                Set<T>(key, value);
                return (T)Convert.ChangeType(value, typeof(T));
            }
        }

        public void Set(string key, string value)
        {
            if (settings.ContainsKey(key))
            {
                if (settings[key] != value)
                {
                    settings[key] = value;
                    changed = true;
                }
            }
            else
            {
                settings.Add(key, value);
                changed = true;
            }
        }

        public void Set<T>(string key, T value)
        {
            if (settings.ContainsKey(key))
            {
                if (settings[key] != (string)Convert.ChangeType(value, typeof(string)))
                {
                    settings[key] = (string)Convert.ChangeType(value, typeof(string));
                    changed = true;
                }
            }
            else
            {
                settings.Add(key, (string)Convert.ChangeType(value, typeof(string)));
                changed = true;
            }
        }

        public Rect ConvertToRect(string value)
        {
            string[] args = ConvertToArgs(value);
            return new Rect(Convert.ToSingle(args[0]), Convert.ToSingle(args[1]), Convert.ToSingle(args[2]), Convert.ToSingle(args[3]));
        }

        public string ConvertToString(Rect rectangle)
        {
            return ConvertFromArgs(new string[] { rectangle.x.ToString(), rectangle.y.ToString(), rectangle.width.ToString(), rectangle.height.ToString() });
        }

        public string[] ConvertToArgs(string value)
        {
            string[] args = value.Split(',');

            for (int i = 0; i < args.Length; i++)
            {
                args[i] = args[i].Trim();
            }

            return args;
        }

        public string ConvertFromArgs(string[] args)
        {
            string value = "";

            for (int i = 0; i < args.Length; i++)
            {
                value += args[i];
                if (i < args.Length - 1)
                {
                    value += ", ";
                }
            }

            return value;
        }

        Rect windowPosition = new Rect((UnityEngine.Screen.width / 2) - 250, (UnityEngine.Screen.height / 2) - 250, 500, 500);
        int windowID = new System.Random().Next();
        Vector2 scrollPosition = Vector2.zero;
        bool isDrawing = false;

        public void DrawWindow()
        {
            GUI.skin = HighLogic.Skin;
            windowPosition = GUILayout.Window(windowID, windowPosition, Window, "Kerbal Engineer Redux - Settings Configurator");
        }

        private void Window(int windowId)
        {
            scrollPosition = GUILayout.BeginScrollView(scrollPosition);

            foreach (KeyValuePair<string, string> setting in settings)
            {
                GUILayout.BeginHorizontal();
                if (!setting.Key.StartsWith("_"))
                {
                    GUILayout.BeginVertical();
                    GUILayout.Label(setting.Key, GUILayout.Width(250));
                    GUILayout.EndVertical();
                    GUILayout.BeginVertical();
                    Set(setting.Key, GUILayout.Toggle(Get<bool>(setting.Key), "ENABLED", GUI.skin.button));
                    GUILayout.EndVertical();
                }
                GUILayout.EndHorizontal();
            }

            GUILayout.EndScrollView();
            if (GUILayout.Button("CLOSE"))
            {
                isDrawing = false;
            }
        }

        public bool IsDrawing
        {
            get
            {
                return isDrawing;
            }
            set
            {
                isDrawing = value;
            }
        }
    }
}
